@@ -63,19 +63,6 @@ class Extractor:
 
         self.data["Sample_Name"] = self.data["Filename"]
 
-        # If metadata file is given, check that the sample names are the
-        # same as in the data
-        if self.metadata is not None:
-            if natsorted(list(self.metadata.Sample_Name)) \
-                    != natsorted(list(self.data.Sample_Name.unique())):
-                raise ValueError("The Sample names in the data and metadata "
-                                 "do not correspond. Please check them and "
-                                 "try again.")
-
-            self.metadata.set_index("Sample_Name", inplace=True)
-            self._check_metadata_columns()
-            self.excel_tables.append(("Metadata", self.metadata))
-
         self.data.drop("Filename", axis=1, inplace=True)
         columns = [
             "Compound", "Sample_Name", "Area", "Sample Type",
@@ -85,6 +72,20 @@ class Extractor:
         self.data = self.data[columns]
         self._replace_nf()
         self._split_dataframes_test()
+
+        # If metadata file is given, check that the sample names are the
+        # same as in the data
+        if self.metadata is not None:
+            if natsorted(list(self.metadata.Sample_Name)) \
+                    != natsorted(list(self.sample_data.Sample_Name.unique())):
+                raise ValueError("The Sample names in the data and metadata "
+                                 "do not correspond. Please check them and "
+                                 "try again.")
+
+            self.metadata.set_index("Sample_Name", inplace=True)
+            self._check_metadata_columns()
+            self.excel_tables.append(("Metadata", self.metadata))
+
         self._get_excluded()
 
         self.met_class = met_class
@@ -184,7 +185,7 @@ class Extractor:
             cols.append(f"Norm{i}")
             cols.append(f"Norm{i}_Unit")
         metadata = pd.DataFrame(columns=cols)
-        metadata["Sample_Name"] = natsorted(self.data["Sample_Name"].unique())
+        metadata["Sample_Name"] = natsorted(self.sample_data["Sample_Name"].unique())
         metadata.set_index("Sample_Name", inplace=True)
         metadata["Volume_Unit"] = "µL"
         return metadata
@@ -521,6 +522,9 @@ class Extractor:
         self.normalised_quantities = self.normalised_quantities[columns]
         # define loqs
         self.loq_table = self.normalised_quantities.copy()
+        # Build loq masks
+        lloq_masks = []
+        uloq_masks = []
         for idx in self.loq_table.index:
             lloq_mask = self.concentration_table.loc[idx, :].apply(
                 lambda x: float(x) < self.calib_data.at[idx, "min"]
@@ -528,12 +532,18 @@ class Extractor:
             uloq_mask = self.concentration_table.loc[idx, :].apply(
                 lambda x: float(x) > self.calib_data.at[idx, "max"]
             )
-            self.loq_table.loc[idx, :] = self.loq_table.loc[idx, :].where(
-                ~lloq_mask, other="<LLOQ"
-            )
-            self.loq_table.loc[idx, :] = self.loq_table.loc[idx, :].where(
-                ~uloq_mask, other=">ULOQ"
-            )
+            lloq_masks.append(lloq_mask)
+            uloq_masks.append(uloq_mask)
+        # Transpose dfs because concentrentration tables are wrong way round
+        lloq_mask = pd.concat(lloq_masks, axis=1).transpose()
+        uloq_mask = pd.concat(uloq_masks, axis=1).transpose()
+        # Apply masks
+        self.loq_table = self.loq_table.mask(
+            lloq_mask == 1, "<LLOQ"
+        )
+        self.loq_table = self.loq_table.mask(
+            uloq_mask == 1, ">ULOQ"
+        )
 
     def _handle_conc_norm(self, cols, base_unit):
         """
@@ -1084,17 +1094,3 @@ class QCError(Error):
 
     def __init__(self, message):
         self.message = message
-
-
-if __name__ == "__main__":
-    test = Extractor(
-        r"C:\Users\legregam\Documents\Projets\MSReader\
-        data\20210506_SOKOL_filtres_MC_quant.xlsx",
-        None, "AA")
-    qc_result = test.handle_qc()
-
-    # r"C:\Users\legregam\Documents\Projets\MSReader\data\Calibration
-    # Report.xlsx", r"C:\Users\legregam\Documents\Projets\MSReader\data
-    # \Sample_List_test.xlsx", data.handle_calibration()
-    # data.generate_concentrations_table(True) data.generate_report()
-    # data.get_ratios()
